@@ -46,30 +46,30 @@
 #include "PeerStorage.h"
 #include "Peer.h"
 #include "Option.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "A2STR.h"
-#include "Request.h"
 #include "bencode2.h"
 #include "bittorrent_helper.h"
 #include "wallclock.h"
+#include "uri.h"
 
 namespace aria2 {
 
 DefaultBtAnnounce::DefaultBtAnnounce
 (const SharedHandle<DownloadContext>& downloadContext,
- const Option* option):
-  downloadContext_(downloadContext),
-  trackers_(0),
-  prevAnnounceTimer_(0),
-  interval_(DEFAULT_ANNOUNCE_INTERVAL),
-  minInterval_(DEFAULT_ANNOUNCE_INTERVAL),
-  userDefinedInterval_(0),
-  complete_(0),
-  incomplete_(0),
-  announceList_(bittorrent::getTorrentAttrs(downloadContext)->announceList),
-  option_(option),
-  logger_(LogFactory::getInstance()),
-  randomizer_(SimpleRandomizer::getInstance())
+ const Option* option)
+  : downloadContext_(downloadContext),
+    trackers_(0),
+    prevAnnounceTimer_(0),
+    interval_(DEFAULT_ANNOUNCE_INTERVAL),
+    minInterval_(DEFAULT_ANNOUNCE_INTERVAL),
+    userDefinedInterval_(0),
+    complete_(0),
+    incomplete_(0),
+    announceList_(bittorrent::getTorrentAttrs(downloadContext)->announceList),
+    option_(option),
+    randomizer_(SimpleRandomizer::getInstance()),
+    tcpPort_(0)
 {}
 
 DefaultBtAnnounce::~DefaultBtAnnounce() {
@@ -79,7 +79,7 @@ bool DefaultBtAnnounce::isDefaultAnnounceReady() {
   return
     (trackers_ == 0 &&
      prevAnnounceTimer_.
-     difference(global::wallclock) >= (userDefinedInterval_==0?
+     difference(global::wallclock()) >= (userDefinedInterval_==0?
                                        minInterval_:userDefinedInterval_) &&
      !announceList_.allTiersFailed());
 }
@@ -103,12 +103,17 @@ bool DefaultBtAnnounce::isAnnounceReady() {
     isDefaultAnnounceReady();
 }
 
-static bool uriHasQuery(const std::string& uri)
+namespace {
+bool uriHasQuery(const std::string& uri)
 {
-  Request req;
-  req.setUri(uri);
-  return !req.getQuery().empty();
+  uri_split_result us;
+  if(uri_split(&us, uri.c_str()) == 0) {
+    return (us.field_set & (1 << USR_QUERY)) && us.fields[USR_QUERY].len > 0;
+  } else {
+    return false;
+  }
 }
+} // namespace
 
 std::string DefaultBtAnnounce::getAnnounceUrl() {
   if(isStoppedAnnounceReady()) {
@@ -132,47 +137,49 @@ std::string DefaultBtAnnounce::getAnnounceUrl() {
   } else {
     return A2STR::NIL;
   }
-  unsigned int numWant = 50;
+  int numWant = 50;
   if(!btRuntime_->lessThanMinPeers() || btRuntime_->isHalt()) {
     numWant = 0;
   }
-  TransferStat stat = peerStorage_->calculateStat();
-  uint64_t left =
+  NetStat& stat = downloadContext_->getNetStat();
+  int64_t left =
     pieceStorage_->getTotalLength()-pieceStorage_->getCompletedLength();
+  // Use last 8 bytes of peer ID as a key
+  const size_t keyLen = 8;
   std::string uri = announceList_.getAnnounce();
   uri += uriHasQuery(uri) ? "&" : "?";
-  uri += "info_hash=";
-  uri += util::torrentPercentEncode(bittorrent::getInfoHash(downloadContext_),
-                                    INFO_HASH_LENGTH);
-  uri += "&peer_id=";
-  uri += util::torrentPercentEncode(bittorrent::getStaticPeerId(),
-                                    PEER_ID_LENGTH);
-  uri += "&uploaded=";
-  uri += util::uitos(stat.getSessionUploadLength());
-  uri += "&downloaded=";
-  uri += util::uitos(stat.getSessionDownloadLength());
-  uri += "&left=";
-  uri += util::uitos(left);
-  uri += "&compact=1";
-  uri += "&key=";
-  // Use last 8 bytes of peer ID as a key
-  size_t keyLen = 8;
-  uri += util::torrentPercentEncode
-    (bittorrent::getStaticPeerId()+PEER_ID_LENGTH-keyLen, keyLen);
-  uri += "&numwant=";
-  uri += util::uitos(numWant);
-  uri += "&no_peer_id=1";
-  if(btRuntime_->getListenPort() > 0) {
-    uri += "&port=";
-    uri += util::uitos(btRuntime_->getListenPort());
+  uri += fmt("info_hash=%s&"
+             "peer_id=%s&"
+             "uploaded=%" PRId64 "&"
+             "downloaded=%" PRId64 "&"
+             "left=%" PRId64 "&"
+             "compact=1&"
+             "key=%s&"
+             "numwant=%d&"
+             "no_peer_id=1",
+             util::torrentPercentEncode
+             (bittorrent::getInfoHash(downloadContext_),
+              INFO_HASH_LENGTH).c_str(),
+             util::torrentPercentEncode
+             (bittorrent::getStaticPeerId(), PEER_ID_LENGTH).c_str(),
+             stat.getSessionUploadLength(),
+             stat.getSessionDownloadLength(),
+             left,
+             util::torrentPercentEncode
+             (bittorrent::getStaticPeerId()+PEER_ID_LENGTH-keyLen,
+              keyLen).c_str(),
+             numWant);
+  if(tcpPort_) {
+    uri += fmt("&port=%u", tcpPort_);
   }
-  std::string event = announceList_.getEventString();
-  if(!event.empty()) {
+  const char* event = announceList_.getEventString();
+  if(event[0]) {
     uri += "&event=";
     uri += event;
   }
   if(!trackerId_.empty()) {
-    uri += "&trackerid="+util::torrentPercentEncode(trackerId_);
+    uri += "&trackerid=";
+    uri += util::torrentPercentEncode(trackerId_);
   }
   if(option_->getAsBool(PREF_BT_REQUIRE_CRYPTO)) {
     uri += "&requirecrypto=1";
@@ -205,7 +212,7 @@ bool DefaultBtAnnounce::isAllAnnounceFailed() {
 }
 
 void DefaultBtAnnounce::resetAnnounce() {
-  prevAnnounceTimer_ = global::wallclock;
+  prevAnnounceTimer_ = global::wallclock();
   announceList_.resetTier();
 }
 
@@ -213,66 +220,54 @@ void
 DefaultBtAnnounce::processAnnounceResponse(const unsigned char* trackerResponse,
                                            size_t trackerResponseLength)
 {
-  if(logger_->debug()) {
-    logger_->debug("Now processing tracker response.");
-  }
+  A2_LOG_DEBUG("Now processing tracker response.");
   SharedHandle<ValueBase> decodedValue =
     bencode2::decode(trackerResponse, trackerResponseLength);
-  const Dict* dict = asDict(decodedValue);
+  const Dict* dict = downcast<Dict>(decodedValue);
   if(!dict) {
     throw DL_ABORT_EX(MSG_NULL_TRACKER_RESPONSE);
   }
-  const String* failure = asString(dict->get(BtAnnounce::FAILURE_REASON));
+  const String* failure = downcast<String>(dict->get(BtAnnounce::FAILURE_REASON));
   if(failure) {
     throw DL_ABORT_EX
-      (StringFormat(EX_TRACKER_FAILURE, failure->s().c_str()).str());
+      (fmt(EX_TRACKER_FAILURE, failure->s().c_str()));
   }
-  const String* warn = asString(dict->get(BtAnnounce::WARNING_MESSAGE));
+  const String* warn = downcast<String>(dict->get(BtAnnounce::WARNING_MESSAGE));
   if(warn) {
-    logger_->warn(MSG_TRACKER_WARNING_MESSAGE, warn->s().c_str());
+    A2_LOG_WARN(fmt(MSG_TRACKER_WARNING_MESSAGE, warn->s().c_str()));
   }
-  const String* tid = asString(dict->get(BtAnnounce::TRACKER_ID));
+  const String* tid = downcast<String>(dict->get(BtAnnounce::TRACKER_ID));
   if(tid) {
     trackerId_ = tid->s();
-    if(logger_->debug()) {
-      logger_->debug("Tracker ID:%s", trackerId_.c_str());
-    }
+    A2_LOG_DEBUG(fmt("Tracker ID:%s", trackerId_.c_str()));
   }
-  const Integer* ival = asInteger(dict->get(BtAnnounce::INTERVAL));
+  const Integer* ival = downcast<Integer>(dict->get(BtAnnounce::INTERVAL));
   if(ival && ival->i() > 0) {
     interval_ = ival->i();
-    if(logger_->debug()) {
-      logger_->debug("Interval:%d", interval_);
-    }
+    A2_LOG_DEBUG(fmt("Interval:%ld", static_cast<long int>(interval_)));
   }
-  const Integer* mival = asInteger(dict->get(BtAnnounce::MIN_INTERVAL));
+  const Integer* mival = downcast<Integer>(dict->get(BtAnnounce::MIN_INTERVAL));
   if(mival && mival->i() > 0) {
     minInterval_ = mival->i();
-    if(logger_->debug()) {
-      logger_->debug("Min interval:%d", minInterval_);
-    }
+    A2_LOG_DEBUG(fmt("Min interval:%ld", static_cast<long int>(minInterval_)));
     minInterval_ = std::min(minInterval_, interval_);
   } else {
     // Use interval as a minInterval if minInterval is not supplied.
     minInterval_ = interval_;
   }
-  const Integer* comp = asInteger(dict->get(BtAnnounce::COMPLETE));
+  const Integer* comp = downcast<Integer>(dict->get(BtAnnounce::COMPLETE));
   if(comp) {
     complete_ = comp->i();
-    if(logger_->debug()) {
-      logger_->debug("Complete:%d", complete_);
-    }
+    A2_LOG_DEBUG(fmt("Complete:%d", complete_));
   }
-  const Integer* incomp = asInteger(dict->get(BtAnnounce::INCOMPLETE));
+  const Integer* incomp = downcast<Integer>(dict->get(BtAnnounce::INCOMPLETE));
   if(incomp) {
     incomplete_ = incomp->i();
-    if(logger_->debug()) {
-      logger_->debug("Incomplete:%d", incomplete_);
-    }
+    A2_LOG_DEBUG(fmt("Incomplete:%d", incomplete_));
   }
   const SharedHandle<ValueBase>& peerData = dict->get(BtAnnounce::PEERS);
-  if(peerData.isNull()) {
-    logger_->info(MSG_NO_PEER_LIST_RECEIVED);
+  if(!peerData) {
+    A2_LOG_INFO(MSG_NO_PEER_LIST_RECEIVED);
   } else {
     if(!btRuntime_->isHalt() && btRuntime_->lessThanMinPeers()) {
       std::vector<SharedHandle<Peer> > peers;
@@ -281,8 +276,8 @@ DefaultBtAnnounce::processAnnounceResponse(const unsigned char* trackerResponse,
     }
   }
   const SharedHandle<ValueBase>& peer6Data = dict->get(BtAnnounce::PEERS6);
-  if(peer6Data.isNull()) {
-    logger_->info("No peers6 received.");
+  if(!peer6Data) {
+    A2_LOG_INFO("No peers6 received.");
   } else {
     if(!btRuntime_->isHalt() && btRuntime_->lessThanMinPeers()) {
       std::vector<SharedHandle<Peer> > peers;
@@ -302,22 +297,24 @@ void DefaultBtAnnounce::shuffleAnnounce() {
   announceList_.shuffle();
 }
 
-void DefaultBtAnnounce::setRandomizer(const RandomizerHandle& randomizer)
+void DefaultBtAnnounce::setRandomizer
+(const SharedHandle<Randomizer>& randomizer)
 {
   randomizer_ = randomizer;
 }
 
-void DefaultBtAnnounce::setBtRuntime(const BtRuntimeHandle& btRuntime)
+void DefaultBtAnnounce::setBtRuntime(const SharedHandle<BtRuntime>& btRuntime)
 {
   btRuntime_ = btRuntime;
 }
 
-void DefaultBtAnnounce::setPieceStorage(const PieceStorageHandle& pieceStorage)
+void DefaultBtAnnounce::setPieceStorage(const SharedHandle<PieceStorage>& pieceStorage)
 {
   pieceStorage_ = pieceStorage;
 }
 
-void DefaultBtAnnounce::setPeerStorage(const PeerStorageHandle& peerStorage)
+void DefaultBtAnnounce::setPeerStorage
+(const SharedHandle<PeerStorage>& peerStorage)
 {
   peerStorage_ = peerStorage;
 }
