@@ -55,7 +55,7 @@ const size_t DEFAULT_MAX_DROPPED_PEER = 50;
 
 } // namespace
 
-const std::string UTPexExtensionMessage::EXTENSION_NAME = "ut_pex";
+const char UTPexExtensionMessage::EXTENSION_NAME[] = "ut_pex";
 
 UTPexExtensionMessage::UTPexExtensionMessage(uint8_t extensionMessageID):
   extensionMessageID_(extensionMessageID),
@@ -151,8 +151,7 @@ bool UTPexExtensionMessage::freshPeersAreFull() const
 bool UTPexExtensionMessage::addDroppedPeer(const SharedHandle<Peer>& peer)
 {
   if(!peer->isIncomingPeer() &&
-     peer->getBadConditionStartTime().
-     difference(global::wallclock()) < interval_) {
+     peer->getDropStartTime().difference(global::wallclock()) < interval_) {
     droppedPeers_.push_back(peer);
     return true;
   } else {
@@ -181,15 +180,15 @@ void UTPexExtensionMessage::setPeerStorage
   peerStorage_ = peerStorage;
 }
 
-UTPexExtensionMessageHandle
+UTPexExtensionMessage*
 UTPexExtensionMessage::create(const unsigned char* data, size_t len)
 {
   if(len < 1) {
     throw DL_ABORT_EX(fmt(MSG_TOO_SMALL_PAYLOAD_SIZE,
-                          EXTENSION_NAME.c_str(),
+                          EXTENSION_NAME,
                           static_cast<unsigned long>(len)));
   }
-  UTPexExtensionMessageHandle msg(new UTPexExtensionMessage(*data));
+  UTPexExtensionMessage* msg(new UTPexExtensionMessage(*data));
 
   SharedHandle<ValueBase> decoded = bencode2::decode(data+1, len - 1);
   const Dict* dict = downcast<Dict>(decoded);
