@@ -12,6 +12,7 @@
 #include "array_fun.h"
 #include "TestUtil.h"
 #include "DiskWriter.h"
+#include "WrDiskCacheEntry.h"
 
 namespace aria2 {
 
@@ -24,6 +25,7 @@ class MultiDiskAdaptorTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testSize);
   CPPUNIT_TEST(testUtime);
   CPPUNIT_TEST(testResetDiskWriterEntries);
+  CPPUNIT_TEST(testWriteCache);
   CPPUNIT_TEST_SUITE_END();
 private:
   SharedHandle<MultiDiskAdaptor> adaptor;
@@ -39,6 +41,7 @@ public:
   void testSize();
   void testUtime();
   void testResetDiskWriterEntries();
+  void testWriteCache();
 };
 
 
@@ -46,15 +49,15 @@ CPPUNIT_TEST_SUITE_REGISTRATION( MultiDiskAdaptorTest );
 
 std::vector<SharedHandle<FileEntry> > createEntries() {
   SharedHandle<FileEntry> array[] = {
-    SharedHandle<FileEntry>(new FileEntry("file0.txt", 0, 0)),
-    SharedHandle<FileEntry>(new FileEntry("file1.txt", 15, 0)),
-    SharedHandle<FileEntry>(new FileEntry("file2.txt", 7, 15)),
-    SharedHandle<FileEntry>(new FileEntry("file3.txt", 0, 22)),
-    SharedHandle<FileEntry>(new FileEntry("file4.txt", 2, 22)),
-    SharedHandle<FileEntry>(new FileEntry("file5.txt", 0, 24)),
-    SharedHandle<FileEntry>(new FileEntry("file6.txt", 3, 24)),
-    SharedHandle<FileEntry>(new FileEntry("file7.txt", 0, 27)),
-    SharedHandle<FileEntry>(new FileEntry("file8.txt", 2, 27)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file0.txt", 0, 0)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file1.txt", 15, 0)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file2.txt", 7, 15)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file3.txt", 0, 22)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file4.txt", 2, 22)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file5.txt", 0, 24)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file6.txt", 3, 24)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file7.txt", 0, 27)),
+    SharedHandle<FileEntry>(new FileEntry(A2_TEST_OUT_DIR"/file8.txt", 2, 27)),
   };
   //           1    1    2    2    3
   // 0....5....0....5....0....5....0
@@ -68,8 +71,7 @@ std::vector<SharedHandle<FileEntry> > createEntries() {
   //                         *** file6
   //                            |file7
   //                            ** file8
-  std::vector<SharedHandle<FileEntry> > entries(&array[0],
-                                                &array[arrayLength(array)]);
+  std::vector<SharedHandle<FileEntry> > entries(vbegin(array), vend(array));
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i = entries.begin();
       i != entries.end(); ++i) {
     File((*i)->getPath()).remove();
@@ -84,15 +86,15 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
-    
+
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -102,16 +104,16 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
-    
+
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     // Because entries[1] spans entries[0]
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -122,17 +124,17 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
-    
+
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(entries[0]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[0]->getDiskWriter());
     // Because entries[2] spans entries[1]
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
     CPPUNIT_ASSERT(entries[1]->needsFileAllocation());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -142,17 +144,17 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
-    
+
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
     // Because entries[4] spans entries[3]
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
     CPPUNIT_ASSERT(entries[3]->needsFileAllocation());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -162,16 +164,16 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
-    
+
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
     // entries[3] is 0 length. No overrap with entries[4]
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -182,15 +184,15 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
-    
+
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -202,15 +204,15 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
-    
+
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -223,12 +225,12 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->openFile();
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -241,14 +243,14 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->openFile();
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter());
     // entries[1] spans entries[2]
-    CPPUNIT_ASSERT(!entries[2]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter());
     CPPUNIT_ASSERT(!entries[2]->needsFileAllocation());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[5]->getDiskWriter());
 
     adaptor->closeFile();
   }
@@ -262,17 +264,17 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->openFile();
     std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
-    CPPUNIT_ASSERT(entries[0]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[1]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[2]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[0]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[1]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[2]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[3]->getDiskWriter());
+    CPPUNIT_ASSERT(!entries[4]->getDiskWriter());
     // entries[6] spans entries[5] in the current implementation.
-    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[6]->getDiskWriter().isNull());
-    CPPUNIT_ASSERT(!entries[7]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[5]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[6]->getDiskWriter());
+    CPPUNIT_ASSERT(entries[7]->getDiskWriter());
     // entries[6] spans entries[8]
-    CPPUNIT_ASSERT(!entries[8]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[8]->getDiskWriter());
     adaptor->closeFile();
   }
 }
@@ -298,9 +300,9 @@ void MultiDiskAdaptorTest::testWriteData() {
   adaptor->writeData((const unsigned char*)msg.c_str(), msg.size(), 0);
   adaptor->closeFile();
 
-  CPPUNIT_ASSERT(File("file0.txt").isFile());
+  CPPUNIT_ASSERT(File(A2_TEST_OUT_DIR"/file0.txt").isFile());
   char buf[128];
-  readFile("file1.txt", buf, 5);
+  readFile(A2_TEST_OUT_DIR"/file1.txt", buf, 5);
   buf[5] = '\0';
   CPPUNIT_ASSERT_EQUAL(msg, std::string(buf));
 
@@ -309,10 +311,10 @@ void MultiDiskAdaptorTest::testWriteData() {
   adaptor->writeData((const unsigned char*)msg2.c_str(), msg2.size(), 5);
   adaptor->closeFile();
 
-  readFile("file1.txt", buf, 15);
+  readFile(A2_TEST_OUT_DIR"/file1.txt", buf, 15);
   buf[15] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("1234567890ABCDE"), std::string(buf));
-  readFile("file2.txt", buf, 1);
+  readFile(A2_TEST_OUT_DIR"/file2.txt", buf, 1);
   buf[1] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("F"), std::string(buf));
 
@@ -321,33 +323,33 @@ void MultiDiskAdaptorTest::testWriteData() {
   adaptor->writeData((const unsigned char*)msg3.c_str(), msg3.size(), 10);
   adaptor->closeFile();
 
-  readFile("file1.txt", buf, 15);
+  readFile(A2_TEST_OUT_DIR"/file1.txt", buf, 15);
   buf[15] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("123456789012345"), std::string(buf));
-  readFile("file2.txt", buf, 7);
+  readFile(A2_TEST_OUT_DIR"/file2.txt", buf, 7);
   buf[7] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("1234567"), std::string(buf));
 
-  CPPUNIT_ASSERT(File("file3.txt").isFile());
+  CPPUNIT_ASSERT(File(A2_TEST_OUT_DIR"/file3.txt").isFile());
 
-  readFile("file4.txt", buf, 2);
+  readFile(A2_TEST_OUT_DIR"/file4.txt", buf, 2);
   buf[2] = '\0';
   CPPUNIT_ASSERT_EQUAL(std::string("12"), std::string(buf));
 
-  CPPUNIT_ASSERT(File("file5.txt").isFile());
+  CPPUNIT_ASSERT(File(A2_TEST_OUT_DIR"/file5.txt").isFile());
 }
 
 void MultiDiskAdaptorTest::testReadData() {
-  SharedHandle<FileEntry> entry1(new FileEntry("file1r.txt", 15, 0));
-  SharedHandle<FileEntry> entry2(new FileEntry("file2r.txt", 7, 15));
-  SharedHandle<FileEntry> entry3(new FileEntry("file3r.txt", 3, 22));
+  SharedHandle<FileEntry> entry1(new FileEntry(A2_TEST_DIR"/file1r.txt", 15, 0));
+  SharedHandle<FileEntry> entry2(new FileEntry(A2_TEST_DIR"/file2r.txt", 7, 15));
+  SharedHandle<FileEntry> entry3(new FileEntry(A2_TEST_DIR"/file3r.txt", 3, 22));
   std::vector<SharedHandle<FileEntry> > entries;
   entries.push_back(entry1);
   entries.push_back(entry2);
   entries.push_back(entry3);
 
   adaptor->setFileEntries(entries.begin(), entries.end());
-
+  adaptor->enableReadOnly();
   adaptor->openFile();
   unsigned char buf[128];
   adaptor->readData(buf, 15, 0);
@@ -366,60 +368,60 @@ void MultiDiskAdaptorTest::testReadData() {
 
 void MultiDiskAdaptorTest::testCutTrailingGarbage()
 {
-  std::string dir = "/tmp";
+  std::string dir = A2_TEST_OUT_DIR;
   std::string prefix = "aria2_MultiDiskAdaptorTest_testCutTrailingGarbage_";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 256, 0)),
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"2", 512, 256))
   };
-  for(size_t i = 0; i < arrayLength(entries); ++i) {
+  for(size_t i = 0; i < A2_ARRAY_LEN(entries); ++i) {
     createFile(entries[i]->getPath(), entries[i]->getLength()+100);
   }
   std::vector<SharedHandle<FileEntry> > fileEntries
-    (&entries[0], &entries[arrayLength(entries)]);
-  
+    (vbegin(entries), vend(entries));
+
   MultiDiskAdaptor adaptor;
   adaptor.setFileEntries(fileEntries.begin(), fileEntries.end());
   adaptor.setMaxOpenFiles(1);
   adaptor.setPieceLength(1);
-  
+
   adaptor.openFile();
 
   adaptor.cutTrailingGarbage();
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)256,
+  CPPUNIT_ASSERT_EQUAL((int64_t)256,
                        File(entries[0]->getPath()).size());
-  CPPUNIT_ASSERT_EQUAL((uint64_t)512,
+  CPPUNIT_ASSERT_EQUAL((int64_t)512,
                        File(entries[1]->getPath()).size());
 }
 
 void MultiDiskAdaptorTest::testSize()
 {
-  std::string dir = "/tmp";
+  std::string dir = A2_TEST_OUT_DIR;
   std::string prefix = "aria2_MultiDiskAdaptorTest_testSize_";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 1, 0)),
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"2", 1, 1))
   };
-  for(size_t i = 0; i < arrayLength(entries); ++i) {
+  for(size_t i = 0; i < A2_ARRAY_LEN(entries); ++i) {
     createFile(entries[i]->getPath(), entries[i]->getLength());
   }
   std::vector<SharedHandle<FileEntry> > fileEntries
-    (&entries[0], &entries[arrayLength(entries)]);
-  
+    (vbegin(entries), vend(entries));
+
   MultiDiskAdaptor adaptor;
   adaptor.setFileEntries(fileEntries.begin(), fileEntries.end());
   adaptor.setMaxOpenFiles(1);
   adaptor.setPieceLength(1);
-  
+
   adaptor.openFile();
 
-  CPPUNIT_ASSERT_EQUAL((uint64_t)2, adaptor.size());
+  CPPUNIT_ASSERT_EQUAL((int64_t)2, adaptor.size());
 }
 
 void MultiDiskAdaptorTest::testUtime()
 {
-  std::string storeDir = "/tmp/aria2_MultiDiskAdaptorTest_testUtime";
+  std::string storeDir = A2_TEST_OUT_DIR"/aria2_MultiDiskAdaptorTest_testUtime";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(storeDir+"/requested", 0, 0)),
     SharedHandle<FileEntry>(new FileEntry(storeDir+"/notFound", 0, 0)),
@@ -435,15 +437,15 @@ void MultiDiskAdaptorTest::testUtime()
   entries[2]->setRequested(false);
 
   std::vector<SharedHandle<FileEntry> > fileEntries
-    (&entries[0], &entries[arrayLength(entries)]);
+    (vbegin(entries), vend(entries));
   MultiDiskAdaptor adaptor;
   adaptor.setFileEntries(fileEntries.begin(), fileEntries.end());
 
   time_t atime = (time_t) 100000;
   time_t mtime = (time_t) 200000;
-  
+
   CPPUNIT_ASSERT_EQUAL((size_t)2, adaptor.utime(Time(atime), Time(mtime)));
-  
+
   CPPUNIT_ASSERT_EQUAL((time_t)mtime,
                        File(entries[0]->getPath()).getModifiedTime().getTime());
 
@@ -454,4 +456,46 @@ void MultiDiskAdaptorTest::testUtime()
                  File(entries[2]->getPath()).getModifiedTime().getTime());
 }
 
+void MultiDiskAdaptorTest::testWriteCache()
+{
+  std::string storeDir =
+    A2_TEST_OUT_DIR"/aria2_MultiDiskAdaptorTest_testWriteCache";
+  SharedHandle<FileEntry> entries[] = {
+    SharedHandle<FileEntry>(new FileEntry(storeDir+"/file1", 16385, 0)),
+    SharedHandle<FileEntry>(new FileEntry(storeDir+"/file2", 4098, 16385))
+  };
+  for(int i = 0; i < 2; ++i) {
+    File(entries[i]->getPath()).remove();
+  }
+  SharedHandle<MultiDiskAdaptor> adaptor(new MultiDiskAdaptor());
+  adaptor->setFileEntries(vbegin(entries), vend(entries));
+  WrDiskCacheEntry cache(adaptor);
+  std::string data1(16383, '1'), data2(100, '2'), data3(4000, '3');
+  cache.cacheData(createDataCell(0, data1.c_str()));
+  cache.cacheData(createDataCell(data1.size(), data2.c_str()));
+  cache.cacheData(createDataCell(data1.size()+data2.size(), data3.c_str()));
+  adaptor->openFile();
+  adaptor->writeCache(&cache);
+  for(int i = 0; i < 2; ++i) {
+    CPPUNIT_ASSERT_EQUAL(entries[i]->getLength(),
+                         File(entries[i]->getPath()).size());
+  }
+  CPPUNIT_ASSERT_EQUAL(data1+data2.substr(0, 2),
+                       readFile(entries[0]->getPath()));
+  CPPUNIT_ASSERT_EQUAL(data2.substr(2)+data3,
+                       readFile(entries[1]->getPath()));
+
+  adaptor->closeFile();
+  for(int i = 0; i < 2; ++i) {
+    File(entries[i]->getPath()).remove();
+  }
+  cache.clear();
+  cache.cacheData(createDataCell(123, data2.c_str()));
+  adaptor->openFile();
+  adaptor->writeCache(&cache);
+  CPPUNIT_ASSERT_EQUAL((int64_t)(123+data2.size()),
+                       File(entries[0]->getPath()).size());
+  CPPUNIT_ASSERT_EQUAL(data2, readFile(entries[0]->getPath()).substr(123));
+}
+
 } // namespace aria2
